@@ -1,18 +1,21 @@
 import warnings
-from typing import Tuple
-
+from typing import Callable, Optional, Tuple, Sequence
 import numpy
-from dask.array.core import Array
 
-from .UnaryIsometricOpWithArgs import OP, _choose_operator
-from .utils import _create_dask_array
+from .DelayedOp import DelayedOp
+from ._isometric import ISOMETRIC_OP_WITH_ARGS, _execute
+from .extract_dense_array import extract_dense_array, _sanitize_to_fortran
+from .extract_sparse_array import extract_sparse_array
+from .create_dask_array import create_dask_array
+from .chunk_shape import chunk_shape
+from .is_sparse import is_sparse
 
 __author__ = "ltla"
 __copyright__ = "ltla"
 __license__ = "MIT"
 
 
-class BinaryIsometricOp:
+class BinaryIsometricOp(DelayedOp):
     """Binary isometric operation involving two n-dimensional seed arrays with the same dimension extents.
     This is based on Bioconductor's ``DelayedArray::DelayedNaryIsoOp`` class.
 
@@ -36,22 +39,22 @@ class BinaryIsometricOp:
             String specifying the operation.
     """
 
-    def __init__(self, left, right, operation: OP):
+    def __init__(self, left, right, operation: ISOMETRIC_OP_WITH_ARGS):
         if left.shape != right.shape:
             raise ValueError("'left' and 'right' shapes should be the same")
 
-        f = _choose_operator(operation)
-        ldummy = numpy.zeros(0, dtype=left.dtype)
-        rdummy = numpy.zeros(0, dtype=right.dtype)
+        ldummy = numpy.zeros(1, dtype=left.dtype)
+        rdummy = numpy.zeros(1, dtype=right.dtype)
         with warnings.catch_warnings():  # silence warnings from divide by zero.
             warnings.simplefilter("ignore")
-            dummy = f(ldummy, rdummy)
+            dummy = _execute(ldummy, rdummy, operation)
         dtype = dummy.dtype
 
         self._left = left
         self._right = right
         self._op = operation
         self._dtype = dtype
+        self._sparse = is_sparse(self._left) and is_sparse(self._right) and dummy[0] == 0
 
     @property
     def shape(self) -> Tuple[int, ...]:
@@ -74,15 +77,6 @@ class BinaryIsometricOp:
         """
         return self._dtype
 
-    def as_dask_array(self) -> Array:
-        """Create a dask array containing the delayed operation.
-
-        Returns:
-            Array: dask array with the delayed subset.
-        """
-        f = _choose_operator(self._op)
-        return f(_create_dask_array(self._left), _create_dask_array(self._right))
-
     @property
     def left(self):
         """Get the left operand satisfying the seed contract.
@@ -109,3 +103,54 @@ class BinaryIsometricOp:
             str: Name of the operation.
         """
         return self._op
+
+ 
+def _extract_array(x: BinaryIsometricOp, subset: Optional[Tuple[Sequence[int]]], f: Callable):
+    ls = f(x._left, subset)
+    rs = f(x._right, subset)
+    return _execute(ls, rs, x._op)
+
+
+@extract_dense_array.register
+def extract_dense_array_BinaryIsometricOp(x: BinaryIsometricOp, subset: Optional[Tuple[Sequence[int]]] = None):
+    """See :py:meth:`~delayedarray.extract_dense_array.extract_dense_array`."""
+    out = _extract_array(x, subset, extract_dense_array)
+    return _sanitize_to_fortran(out)
+
+
+@extract_sparse_array.register
+def extract_sparse_array_BinaryIsometricOp(x: BinaryIsometricOp, subset: Optional[Tuple[Sequence[int]]] = None):
+    """See :py:meth:`~delayedarray.extract_sparse_array.extract_sparse_array`."""
+    return _extract_array(x, subset, extract_sparse_array)
+
+
+@create_dask_array.register
+def create_dask_array_BinaryIsometricOp(x: BinaryIsometricOp): 
+    """See :py:meth:`~delayedarray.create_dask_array.create_dask_array`."""
+    ls = create_dask_array(x._left)
+    rs = create_dask_array(x._right)
+    return _execute(ls, rs, x._op)
+
+
+@chunk_shape.register
+def chunk_shape_BinaryIsometricOp(x: BinaryIsometricOp):
+    """See :py:meth:`~delayedarray.chunk_shape.chunk_shape`."""
+    lchunk = chunk_shape(x._left)
+    rchunk = chunk_shape(x._right)
+
+    # Not bothering with taking the lowest common denominator, as that
+    # might be too aggressive and expanding to the entire matrix size.
+    # We instead use the maximum chunk size (which might also expand, e.g.,
+    # if you're combining column-major and row-major matrices; oh well).
+    # Just accept that we'll probably need to break chunks during iteration.
+    output = []
+    for i in range(len(lchunk)):
+        output.append(max(lchunk[i], rchunk[i]))
+
+    return (*output,) 
+
+
+@is_sparse.register
+def is_sparse_BinaryIsometricOp(x: BinaryIsometricOp):
+    """See :py:meth:`~delayedarray.is_sparse.is_sparse`."""
+    return x._sparse
