@@ -18,10 +18,19 @@ finally:
 from .BinaryIsometricOp import BinaryIsometricOp
 from .Cast import Cast
 from .Combine import Combine
+from .DelayedOp import DelayedOp
 from .DelayedArray import DelayedArray
 from .Round import Round
 from .Subset import Subset
 from .Transpose import Transpose
 from .UnaryIsometricOpSimple import UnaryIsometricOpSimple
 from .UnaryIsometricOpWithArgs import UnaryIsometricOpWithArgs
-from .utils import extract_array, create_dask_array, chunk_shape, guess_iteration_block_size
+from .SparseNdarray import SparseNdarray
+
+from .extract_dense_array import extract_dense_array
+from .extract_sparse_array import extract_sparse_array
+from .create_dask_array import create_dask_array
+from .is_sparse import is_sparse
+from .chunk_shape import chunk_shape
+from .is_pristine import is_pristine
+from .guess_iteration_block_size import guess_iteration_block_size
